# Copyright 2023 AUI, Inc. Washington DC, USA
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.

def print_log_messages(msg, logger, is_err=False, verbose=2):
    """
    Print msg and optionally write it to an instance of the casalogger.

    If msg is a list then the elements are each printed first followed by logging each element.

    Messages are normally printed to sys.stdout and logged as INFO to the casalogger.
    
    When is_err is True the message is printed sys.stderr and logged as SEVERE

    When verbose is < 2 then the message is not printed unless there is no logger

    This function is intended for internal casaconfig use.

    Parameters
       - msg (str) - The message to print and optionally log.
       - logger (casatools.logsink) - Instance of the casalogger to use. Not used if None.
       - is_err (bool=False) - When False, output goes to sys.stdout and logged as INFO level. When True, output goes to sys.stderr and logged as SEVERE
       - verbose (int=2) - When < 2 then msg is only printed if there is no logger, otherwise it's just logged

    Returns
       None

    """

    import sys
    fileout = sys.stdout
    loglevel = 'INFO'
    if (is_err):
        fileout = sys.stderr
        loglevel = 'SEVERE'

    # this is rarely called and this should be a fast operation, it makes the code simpler
    if type(msg) is not list:
        msg = [msg]

    # always print if there is no logger, if there is a logger and verbose is < 2 then do not print
    if (logger is None) or (not verbose < 2):
        for m_msg in msg:
            print(m_msg,file=fileout)

    for m_msg in msg:
        if logger is not None: logger.post(m_msg, loglevel)



# Copyright 2023 AUI, Inc. Washington DC, USA
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
"""
this module will be included in the api
"""

def get_data_info(path=None, logger=None, type=None):
    """
    Get the summary information on the 3 types of data managed by casaconfig.

    The return value is a dictionary by type of data : 'casarundata', 'measures',
    and 'release'. Each type is itself a dictionary as described below.

    The path is the location to use to search for the installed release information.
    The path argument defaults to config.measurespath when not set.

    The return value can be limited to just a specific type using the type
    argument ('casarundata', 'measures', 'release'). In that case the returned value
    is the dictionary for just the requested type. When type is None (the default)
    the full set of dictionaries is returned.

    The 'casarundata' and 'measures' dictionaries contain 'version' 
    and 'date' where version is the version string and date is the date when it  
    was installed. These values are taken from the readme.txt file for each type.
    For 'casarundata' an additional field of 'manifest' is present which is
    the list of files that have been installed for that specific version (this will
    be empty for an unknown or invalid version).

    The 'release' dictionary comes from the release_data_readme.txt file which is copied
    into place when a modular CASA is built. It consists of 'casarundata' and 'measures' 
    dictionaries where each dictionary contains the 'version' string and 'date' that that
    version was created for the described release. This allows casaconfig to install that
    casarundata version for testing purposes. The release information does not depend 
    on the path argument since it is found in the casaconfig module.

    If path is empty or does not exist then the return value for the 'casarundata' and
    'measures' types is None.

    If no readme.txt file can be found at path but the contents of path have the directories
    expected for casarundata then the version returned for 'casarundata' is 'unknown' and
    the date is an empty string. In that case the path may contain casarundata from a legacy
    installation of CASA data. CASA will be able to use the files at this location but they
    can not be maintained by casaconfig. If the path is not empty (except for a possible lock
    file while it's in use but does not appear to contain legacy casarundata or the readme.txt 
    file found there can not be read as expected then the version is 'invalid'.

    If no readme.txt file can be found for the measures at path/geodetic but both the geodetic
    and ephemeris directories are present in path then the version returned for 'measures' is
    'unknown' and the date is an empty string. The path location may contain measures data from
    a legacy installation of CASA data. CASA will be able to use any measures tables at this
    location by they can not be maintained by casaconfig. If the path is not empty but does not
    appear to contain legacy measures data or the readme.txt file found in path/geodetic can
    not be read as expected then the version is 'invalid'.

    If no casadata release information is found or the contents are unexpected the returned
    value for 'release' is None and the "--reference-testing" option will not do anything
    for this installation of casaconfig. This will be the case for a modular installation.

    If path has not been set (has a value of None) then the returned value will be None. This
    likely means that a casasiteconfig.py exists but has not yet been edited to set measurespath.

    Parameters
       - path (str) - Folder path to find the casarundata and measures data information. If not set then config.measurespath is used.
       - logger (casatools.logsink=None) - Instance of the casalogger to use for writing messages. Default None writes messages to the terminal.
       - type (str) - the specific type of data info to return (None, 'casarundata', 'measures', 'release'; None returns a dictionary of all types)
    
    Returns
       - a dictionary by type, 'casarundata', 'measures', 'release' where each type is a dictionary containing 'version' and 'date'. A return value of None indicates path is unset. A value of None for that type means no information could be found about that type. If a specific type is requested then only the dictionary for that type is returned (or None if that type can not be found).

    Raises
       - casaconfig.UnsetMeasurespath - path is None and has not been set in config
       - ValueError - raised when type has an invalid value

    """

    result = None

    import os
    import time
    import importlib.resources
    from .print_log_messages import print_log_messages
    from .read_readme import read_readme
    
    from casaconfig import UnsetMeasurespath


    currentTime = time.time()
    secondsPerDay = 24. * 60. * 60.

    if path is None:
        from .. import config as _config
        path = _config.measurespath

    if path is None:
        raise UnsetMeasurespath('get_data_info: path is None and has not been set in config.measurespath. Provide a valid path and retry.')
    
    path = os.path.abspath(os.path.expanduser(path))

    result = {'casarundata':None, 'measures':None, 'release':None}
    
    if type is not None and type not in result:
        raise ValueError("invalid type %s; must be one of None, 'casarundata', 'measures', 'release'" % type)

    # casarundata and measures 

    if os.path.isdir(path) and (len(os.listdir(path))>0):
        # if the only thing at path is the lock file then proceed as if path is empty - skip this section
        pathfiles = os.listdir(path)
        if len(pathfiles) == 1 and pathfiles[0] == "data_update.lock":
            pass
        else:
            # there's something at path, look for the casarundata readme
            if type is None or type=='casarundata':
                datareadme_path = os.path.join(path,'readme.txt')
                if os.path.exists(datareadme_path):
                    # the readme exists, get the info
                    result['casarundata'] = {'version':'error', 'date':'', 'manifest':[], 'age':None}
                    readmeContents = read_readme(datareadme_path)
                    if readmeContents is not None:
                        currentAge = (currentTime - os.path.getmtime(datareadme_path)) / secondsPerDay
                        currentVersion = readmeContents['version']
                        currentDate = readmeContents['date']
                        # the manifest ('extra') must exist with at least 1 entry, otherwise this is no a valid readme file and the version should be 'error'
                        if len(readmeContents['extra']) > 0:
                            result['casarundata'] = {'version':currentVersion, 'date':currentDate, 'manifest':readmeContents['extra'], 'age':currentAge}
                else:
                    # does it look like it's probably casarundata?
                    expected_dirs = ['alma','catalogs','demo','ephemerides','geodetic','gui','nrao']
                    ok = True
                    for d in expected_dirs:
                        if not os.path.isdir(os.path.join(path,d)): ok = False
                    if ok:
                        # probably casarundata
                        result['casarundata'] = {'version':'unknown', 'date':'', 'manifest': None,'age':None}
                    else:
                        # probably not casarundata
                        # this is invalid, unexpected things are happening there
                        result['casarundata'] = {'version':'invalid', 'date':'', 'manifest': None, 'age':None}

            if type is None or type=='measures':
                # look for the measures readme
                measuresreadme_path = os.path.join(path,'geodetic/readme.txt')
                if os.path.exists(measuresreadme_path):
                    # the readme exists, get the info
                    result['measures'] = {'version':'error', 'date':'', 'age':None}
                    readmeContents = read_readme(measuresreadme_path)
                    if readmeContents is not None:
                        currentVersion = readmeContents['version']
                        currentDate = readmeContents['date']
                        currentAge = (currentTime - os.path.getmtime(measuresreadme_path)) / secondsPerDay
                        result['measures'] = {'version':currentVersion,'date':currentDate,'age':currentAge}
                else:
                    # does it look like it's probably measuresdata?
                    # path should have ephemerides and geodetic directories
                    if os.path.isdir(os.path.join(path,'ephemerides')) and os.path.isdir(os.path.join(path,'geodetic')):
                        result['measures'] = {'version':'unknown', 'date':'', 'age':None}
                    else:
                        # probably not measuresdata
                        result['measures'] = {'version':'invalid', 'date':'', 'age':None}

    if type is None or type=='release':
        # release data versions
        if importlib.resources.is_resource('casaconfig','release_data_readme.txt'):
            try:
                casarundataVersion = None
                measuresVersion = None
                ok = True
                reason = None
                readme_lines = importlib.resources.read_text('casaconfig','release_data_readme.txt').split('\n')
                for readmeLine in readme_lines:
                    # lines must contain something and not start with #
                    if len(readmeLine) > 0 and readmeLine[0] != '#':
                        splitLine = readmeLine.split(':')
                        if len(splitLine) == 2:
                            lineType = splitLine[0].strip()
                            lineVers = splitLine[1].strip()
                            if lineType == 'casarundata':
                                if casarundataVersion is not None:
                                    ok = False
                                    reason = "duplicate casarundata lines"
                                    break
                                casarundataVersion = lineVers
                            elif lineType == 'measures':
                                if measuresVersion is not None:
                                    ok = False
                                    reason = "duplicate measures lins"
                                    break
                                measuresVersion = lineVers
                            else:
                                ok = False
                                reason = "Unexpected type : %s" % lineType
                                break
                        else:
                            ok = False
                            reason = "Missing or too many ':' separators"
                if (casarundataVersion is None or measuresVersion is None) and ok:
                    ok = False
                    reason = "missing one or more version strings for expected casarundata and measures types"

                if not ok:
                    print_log_messages("Incorrectly formatted release_data_readme.txt. %s" % reason, logger, True)
                    # leave 'release' as None
                else:
                    result['release'] = {'casarundata':casarundataVersion, 'measures':measuresVersion}
            except:
                print("Unexpected error reading release_data_readme.txt")
                # leave 'release' as None
        else:
            # no release information available, probably a modular install only
            # leave 'release' as None
            pass

    if type is not None:
        result = result[type]
        
    return result



# Copyright 2023 AUI, Inc. Washington DC, USA
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.

def do_pull_data(path, version, installed_files, currentVersion, currentDate, logger):
    """
    Pull the casarundata for the given version and install it in path, removing
    the installed files and updating the readme.txt file when done.

    This function is used by both pull_data and data_update when each has
    determind that the desired version should be installed. The calling function
    has already obtained the lock. No additional checking happens here. The
    calling function has already examined any existing readme file and used that
    to set the installed_files list as appropriate. 

    Parameters
       - path (str) - Folder path to place casadata contents.
       - version (str) - casadata version to retrieve.
       - installed_files (str list) - list of installed files from the version already installed. Set to an empty list if there is no previously installed version.
       - currentVersion (str) - from the readme file if it already exists, or an empty string if there is no previously installed version.
       - currentDate (str) - from the readme file if it already exists, or an empty string if there is no previously installed version.
       - logger (casatools.logsink) - Instance of the casalogger to use for writing messages. Messages are always written to the terminal. Set to None to skip writing messages to a logger.

    Returns
       None

    """

    import os
    import sys
    from datetime import datetime
    import ssl
    import urllib.request
    import certifi
    import tarfile
    import shutil

    from .print_log_messages import print_log_messages
    
    readme_path = os.path.join(path, 'readme.txt')

    if (installed_files is not None and len(installed_files) > 0):
        # remove the previously installed files
        # remove this readme file so it's not confusing if something goes wrong after this
        os.remove(readme_path)
        print_log_messages('Removing files using manifest from previous install of %s on %s' % (currentVersion, currentDate), logger)
        for relpath in installed_files:
            filepath = os.path.join(path,relpath)
            # don't say anything if filepath isn't found, remove it if it is found
            if os.path.exists(filepath) and os.path.isfile(filepath):
                os.remove(filepath)
        
        # remove any empty directories in path - this is recursive
        def remove_empty_dirs(dirpath):
            # look at all of the files in dirpath, for dirs, go down that recursively
            # if there's nothing there after the dirs have all been handled, remove it
            files = os.listdir(dirpath)
            not_dirs = []
            for f in os.listdir(dirpath):
                fpath = os.path.join(dirpath, f)
                if os.path.isdir(fpath):
                    remove_empty_dirs(fpath)
                else:
                    not_dirs.append(f)
            if len(not_dirs) == 0:
                if len(os.listdir(dirpath)) == 0:
                    os.rmdir(dirpath)
                    
        remove_empty_dirs(path)

    # okay, safe to install the requested version

    goURL = 'https://go.nrao.edu/casarundata'
    context = ssl.create_default_context(cafile=certifi.where())

    # need to first resolve the go.nrao.edu URL to find the actual data URL
    dataURLroot = urllib.request.urlopen(goURL, context=context).url
    dataURL = os.path.join(dataURLroot, version)
    
    with urllib.request.urlopen(dataURL, context=context, timeout=400) as tstream, tarfile.open(fileobj=tstream, mode='r|*') as tar :
        l = int(tstream.headers.get('content-length', 0))
        sizeString = "unknown size"
        if (l>0): sizeString = ("%.0fM" % (l/(1024*1024)))
        # use print directly to make use of the end argument
        print('downloading casarundata contents to %s (%s) ... ' % (path,sizeString), file = sys.stdout, end="" )
        sys.stdout.flush()
        # also log it
        if logger is not None: logger.post('downloading casarundata contents to %s ...' % path, 'INFO')
        # use the 'data' filter if available, revert to previous 'fully_trusted' behavior of not available
        tar.extraction_filter = getattr(tarfile, 'data_filter', (lambda member, path: member))
        tar.extractall(path=path)
        
    print("done", file=sys.stdout)
        
    # the tarball has been extracted to path/version
    # get the instaled files of files to be written to the readme file
    versdir = os.path.join(path,version[:version.index('.tar')])
    installed_files = []
    wgen = os.walk(versdir)
    for (dirpath, dirnames, filenames) in wgen:
        for f in filenames:
            installed_files.append(os.path.relpath(os.path.join(dirpath,f),versdir))
                
    # move everything in version up a level to path
    for f in os.listdir(versdir):
        srcPath = os.path.join(versdir,f)
        if os.path.isdir(srcPath):
            # directories are first copied, then removed
            # existing directories are reused, existing files are overwritten
            # things in path that do not exist in srcPath are not changed
            shutil.copytree(srcPath,os.path.join(path,f),dirs_exist_ok=True)
            shutil.rmtree(srcPath)
        else:
            # assume it's a simple file, these can be moved directly, overwriting anything already there
            os.rename(srcPath,os.path.join(path,f))
                        
    # safe to remove versdir, it would be a surprise if it's not empty
    os.rmdir(versdir)
    # update the readme.txt file
    with open(readme_path,'w') as fid:
        fid.write("# casarundata populated by casaconfig.pull_data\nversion : %s\ndate : %s" % (version, datetime.today().strftime('%Y-%m-%d')))
        fid.write("\n#\n# manifest")
        for f in installed_files:
            fid.write("\n%s" % f)

    print_log_messages('casarundata installed %s at %s' % (version, path), logger)



# Copyright 2023 AUI, Inc. Washington DC, USA
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
"""
this module will be included in the api
"""

def do_auto_updates(configDict, logger=None, verbose=None):
    """
    Use measurespath, data_auto_update, and measures_auto_update from configDict to
    do any auto updates as necessary.

    This is intended for use during casatools init but may be useful in other cases.

    Note that the IERS measures data is usually read when casatools starts and so 
    changes made to that table may not be seen until a new session if this function
    is used outside of casatools initialization.

    measurespath must be set (not None).

    measures_auto_update must be True when data_auto_update is True.

    See the documentation for data_update and measures_update for additional details
    about the auto update rules.

    The verbose argument controls the level of information provided when this function when the data
    are unchanged for expected reasons. A level of 0 prints and logs nothing. A
    value of 1 logs the information and a value of 2 logs and prints the information.

    See data_update and measures_update for additional details about exceptions

    Paramters
       - configDict (dict) - A config dictionary previously set. 
       - logger (casatools.logsink=None) - Instance of the casalogger to use for writing messages. Default None writes messages to the terminal.
       - verbose (int) - Level of output, 0 is none, 1 is to logger, 2 is to logger and terminal, defaults to casaconfig_verbose in the config dictionary.

    Returns
       None

    Raises
       - casaconfig.UnsetMeasurespath - raised when measurespath is None in config

    """

    from .print_log_messages import print_log_messages
    from .data_update import data_update
    from .measures_update import measures_update

    from casaconfig import UnsetMeasurespath

    if configDict.measurespath is None:
        # continue, because things still might work if there are measures in datapath
        raise UnsetMeasurespath('do_auto_updates: measurespath is None in configDict. Provide a valid path and retry.')

    if verbose is None:
        verbose = configDict.casaconfig_verbose

    if (configDict.measures_auto_update or configDict.data_auto_update):
        if (configDict.data_auto_update and (not configDict.measures_auto_update)):
            print_log_messages('measures_auto_update must be True when data_auto_update is True, skipping auto updates', logger, True)
        else:
            if configDict.data_auto_update:
                data_update(configDict.measurespath, logger=logger, auto_update_rules=True, verbose=verbose)
            if configDict.data_auto_update or configDict.measures_auto_update:
                measures_update(configDict.measurespath, logger=logger, auto_update_rules=True, verbose=verbose)

    return



# Copyright 2023 AUI, Inc. Washington DC, USA
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
"""
this module will be included in the api
"""

def data_available():
    """
    List available casarundata versions on CASA server at https://go.nrao.edu/casarundata

    This returns a list of the casarundata versions available on the CASA
    server. The version parameter of data_update must be one
    of the values in that list if set (otherwise the most recent version
    in this list is used).

    A casarundata version is the filename of the tarball and look 
    like "casarundata.x.y.z.tar.*" (different compressions may be used by CASA without
    changing casaconfig functions that use those tarballs). The full filename is
    the casarundata version expected in casaconfig functions.

    Parameters
       None
    
    Returns
       list - version names returned as list of strings

    Raises
       - casaconfig.RemoteError - Raised when there is an error fetching some remote content
       - Exception - Unexpected exception while getting list of available casarundata versions

    """

    import html.parser
    import urllib.request
    import urllib.error
    import ssl
    import certifi

    from casaconfig import RemoteError
    
    class LinkParser(html.parser.HTMLParser):
        def reset(self):
            super().reset()
            self.rundataList = []

        def handle_starttag(self, tag, attrs):
            if tag == 'a':
                for (name, value) in attrs:
                    # only care if this is an href and the value starts with
                    # casarundata and has '.tar.' somewhere later and does not end in .md5
                    if name == 'href' and (value.startswith('casarundata') and (value.rfind('.tar')>11) and (value[-4:] != '.md5')):
                        # only add it to the list if it's not already there
                        if (value not in self.rundataList):
                            self.rundataList.append(value)

    try:
        context = ssl.create_default_context(cafile=certifi.where())
        with urllib.request.urlopen('https://go.nrao.edu/casarundata/', context=context, timeout=400) as urlstream:
            parser = LinkParser()
            encoding = urlstream.headers.get_content_charset() or 'UTF-8'
            for line in urlstream:
                parser.feed(line.decode(encoding))

        # return the sorted list, earliest versions are first, newest is last
        return sorted(parser.rundataList)

    except urllib.error.URLError as urlerr:
        raise RemoteError("Unable to retrieve list of available casarundata versions : " + str(urlerr)) from None
        
    except Exception as exc:
        msg = "Unexpected exception while getting list of available casarundata versions : " + str(exc)
        raise Exception(msg)

    # nothing to return if it got here, must have been an exception
    return []

    



# Copyright 2024 AUI, Inc. Washington DC, USA
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.

def summary(configDict = None):
    """
    Summarize the configure status.

    This prints out the config files that were loaded successfully, the files are shown by 
    type (default, site, user), the command flags are shown that control whether the site 
    and user config files are used, any errors encountered while loading are printed, the
    value of the measurespath config value is shown, the installed data versions are shown
    (casarundata and measures) and any release version information is shown.

    Parameters
        - configDict (dict) - a config dictionary. If None this is imported from casaconfig.

    Returns
        None

    """

    import casaconfig

    if configDict is None:
        from casaconfig import config as configDict

    try:
        dataInfo = casaconfig.get_data_info()
    except casaconfig.UnsetMeasurespath:
        print("Measurespath is unset or does not exist in config dictionary. Use a valid path and try again")
        return
    
    print("")
    print("casaconfig summary")
    print("")
    print("loaded config files : ")
    for f in configDict.load_success():
        print("   %s" % f)
    if len(configDict.load_failure()) > 0:
        print("")
        print("config failures :")
        for f in configDict.load_failure():
            print("\n%s :" % f)
            print(configDict.load_failure()[f])
    print("")
    if (configDict.__flags.noconfig):
        print("--noconfig option was used")
    if (configDict.__flags.nositeconfig):
        print("--nositeconfig option was used")
    print("")
    print("measurespath : %s" % configDict.measurespath)
    print("")
    if (dataInfo['casarundata'] is None):
        print("casarundata version : no casarundata found")
    else:
        rundataVers = dataInfo['casarundata']['version']
        msg = "casarundata version : %s" % rundataVers
        if rundataVers == "unknown":
            msg += "; legacy data not maintained by casaconfig"
        elif rundataVers == "error":
            msg += "; unexpected readme.txt file, casarundata should be reinstalled"
        print(msg)
    if (dataInfo['measures'] is None):
        print("measures version : no measures found")
    else:
        measVers = dataInfo['measures']['version']
        msg = "measures version : %s" % measVers
        if measVers == "unknown":
            msg += "; legacy data not maintained by casaconfig"
        elif measVers == "error":
            msg += "; unexpected readme.txt file, measures should be reinstalled"
        print(msg)
    
    if (dataInfo['release'] is None):
        print("no release version information found")
    else:
        print('release casarundata version : %s' % dataInfo['release']['casarundata'])
        print('release measures version : %s' % dataInfo['release']['measures'])
    print("")



# Copyright 2020 AUI, Inc. Washington DC, USA
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
"""
this module will be included in the api
"""


def measures_available():
    """
    List available measures versions on the ASTRON server

    This returns a list of the measures files available on the ASTRON
    server. The version parameter of measures_update must be one
    of the values in that list if set (otherwise the most recent version
    in this list is used).

    Parameters
       None
    
    Returns
       version names returned as list of strings

    Raises:
       - casaconfig.RemoteError - raised when when a socket.gaierror is seen, likely due to no network connection
       - Exception: raised when any unexpected exception happens

    """
    from ftplib import FTP
    import socket

    from casaconfig import RemoteError

    files = []
    try:
        ftp = FTP('ftp.astron.nl')
        rc = ftp.login()
        rc = ftp.cwd('outgoing/Measures')
        files = ftp.nlst()
        ftp.quit()
        #files = [ff.replace('WSRT_Measures','').replace('.ztar','').replace('_','') for ff in files]
        files = [ff for ff in files if (len(ff) > 0) and (not ff.endswith('.dat'))]
    except socket.gaierror as gaierr:
        raise RemoteError("Unable to retrieve list of available measures versions : " + str(gaierr)) from None
    except Exception as exc:
        msg = "Unexpected exception while getting list of available measures versions : " + str(exc)
        raise Exception(msg)
        
    return files



import sys
import os
from casaconfig.private.get_argparser import get_argparser

parser = get_argparser(add_help=True)

# get_argparser supplies configfile, noconfig, nositeconfig
# add measurespath, pull-data, data-update, measures-update, update-all, reference-testing, current-data

parser.prog = "casaconfig"

# measurespath will default to the value in config if not set here
parser.add_argument( "--measurespath", dest='measurespath', default=None,
                     help="location of casarundata")

parser.add_argument( "--pull-data", dest='pulldata', action='store_const', const=True, default=False,
                     help="invoke pull_data() to populate measurespath with latest casarundata")
parser.add_argument( "--data-update", dest='dataupdate', action='store_const', const=True, default=False,
                     help="invoke data_update() to update measurespath to the latest casarundata")
parser.add_argument( "--measures-update", dest='measuresupdate', action='store_const', const=True, default=False,
                     help="invoke measures_update() to update measurespath to the latest measures data")
parser.add_argument( "--update-all", dest='updateall', action='store_const', const=True, default=False,
                     help="invoke update_all() to populate (update) measurespath with the latest casarundata and measures data")
parser.add_argument( "--reference-testing", action='store_const', const=True, dest='referencetesting', default=False,
                     help="set measurespath to the casarundata when this version was produced, used for testing purposes")
parser.add_argument( "--current-data", dest='currentdata', action='store_const', const=True, default=False,
                     help="print out a summary of the current casarundata and measures data installed in measurespath and then exit")
parser.add_argument( "--summary", dest='summary', action='store_const', const=True, default=False,
                     help="print out a summary of casaconfig data handling and the exit")
parser.add_argument("--force", dest='force', action='store_const', const=True, default=False,
                    help="force an update using the force=True option to update_all, data_update, and measures_update")

# initialize the configuration to be used
flags,args = parser.parse_known_args(sys.argv)
from casaconfig import config

# import the casaconfig module
import casaconfig

# make sure measurespath reflects any command line value
if flags.measurespath is None:
    flags.measurespath = config.measurespath
else:
    config.measurespath = flags.measurespath

# watch for measurespath of None, that likely means that casasiteconfig.py is in use and this has not been set. It can't be used then.
try:
    if flags.measurespath is None:
        print("measurespath has been set to None in the user or site config file.")
        print("Either provide a measurespath on the casaconfig command line or edit the user or site config file to set measurespath to a location.")
        sys.exit(1)

    # do any expanduser and abspath - this is what should be used
    measurespath = os.path.abspath(os.path.expanduser(flags.measurespath))

    if flags.currentdata:
        if not os.path.exists(measurespath) or not os.path.isdir(measurespath):
            print("No data installed at %s. The measurespath does not exist or is not a directory." % measurespath)
        else:
            print("current data installed at %s" % measurespath)
            dataInfo = casaconfig.get_data_info(measurespath)
        
            # casarundata
            casarunInfo = dataInfo['casarundata']
            if casarunInfo is None or casarunInfo['version'] == "invalid":
                print("No casarundata found (missing readme.txt and not obviously legacy casa data).")
            if casarunInfo['version'] == "error":
                print("Unexpected casarundata readme.txt content; casarundata should be reinstalled.")
            elif casarunInfo['version'] == "unknown":
                print("casarundata version is unknown (probably legacy casa data not maintained by casaconfig).")
            else:
                currentVersion = casarunInfo['version']
                currentDate = casarunInfo['date']
                print('casarundata version %s installed on %s' % (currentVersion, currentDate))
            
            # measures
            measuresInfo = dataInfo['measures']
            if measuresInfo is None or measuresInfo['version'] == "invalid":
                print("No measures data found (missing readme.txt and not obviously legacy measures data).")
            if measuresInfo['version'] == "error":
                print("Unexpected measures readme.txt content; measures should be reinstalled.")
            elif measuresInfo['version'] == "unknown":
                print("measures version is unknown (probably legacy measures data not maintained by casaconfig).")
            else:
                currentVersion = measuresInfo['version']
                currentDate = measuresInfo['date']
                print('measures version %s installed on %s' % (currentVersion, currentDate))
 
        # ignore any other arguments
    elif flags.summary:
        from casaconfig.private.summary import summary
        summary(config)
        # ignore any other arguments
    else:
        if flags.referencetesting:
            print("reference testing using pull_data and 'release' version into %s" % measurespath)
            casaconfig.pull_data(measurespath,'release')
            # ignore all other options
        else:
            # watch for nothing actually set
            if not flags.updateall and not flags.pulldata and not flags.dataupdate and not flags.measuresupdate:
                parser.print_help()
                sys.exit(1)
            # the update options, update all does everything, no need to invoke anything else
            print("Checking for updates into %s" % measurespath)
            if flags.updateall:
                casaconfig.update_all(measurespath,force=flags.force)
            else:
                # do any pull_update first
                if flags.pulldata:
                    casaconfig.pull_data(measurespath)
                # then data_update, not necessary if pull_data just happened
                if flags.dataupdate and not flags.pulldata:
                    casaconfig.data_update(measurespath, force=flags.force)
                # then measures_update
                if flags.measuresupdate:
                    casaconfig.measures_update(measurespath, force=flags.force)

except casaconfig.UnsetMeasurespath as exc:
    # UnsetMeasurespath should not happen because measurespath is checked to not be None above, but just in case
    print(str(exc))
    print("This exception should not happen, check the previous messages for additional information and try a different path")
    sys.exit(1)

except casaconfig.BadReadme as exc:
    print(str(exc))
    sys.exit(1)

except casaconfig.RemoteError as exc:
    print(str(exc))
    print("This is likely due to no network connection or bad connectivity to a remote site, wait and try again")
    sys.exit(1)

except casaconfig.BadLock as exc:
    # the output produced by the update functions is sufficient, just re-echo the exception text itself
    print(str(exc))
    sys.exit(1)

except casaconfig.NotWritable as exc:
    print(str(exc))
    sys.exit(1)

sys.exit(0)



# Copyright 2024 AUI, Inc. Washington DC, USA
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
"""
this module will be included in the api
"""

class AutoUpdatesNotAllowed(Exception):
    """Raised when a path does not exist or is not owned by the user"""
    pass

class BadLock(Exception):
    """Raised when the lock file is not empty and a lock attempt was made"""
    pass

class BadReadme(Exception):
    """Raised when a readme.txt file does not contain the expected contents"""
    pass

class NoReadme(Exception):
    """Raised when the readme.txt file is not found at path (path also may not exist)"""

class RemoteError(Exception):
    """Raised when there is an error fetching some remote content"""
    pass

class NotWritable(Exception):
    """Raised when the path is not writable by the user"""

class UnsetMeasurespath(Exception):
    """Raised when a path argument is None"""
    pass
